//
//  MSCommandsManager.h
//  MSCommandsManager
//
//  Created by Paul Herzog on 19-Jul-2007.
//  Copyright 2007 MacSpeech, Inc. All rights reserved.
//

#import <Cocoa/Cocoa.h>

#import "ActionMO.h"
#import "CommandMO.h"
#import "LocationMO.h"
#import "TriggerMO.h"
#import "MSCommandContext.h"
#import "MSCommandGroup.h"
#import "MSLocale.h"
#import "BDAliasStringTransformer.h"
//#import "AppleScriptStringTransformer.h"

@class MSSystemMonitor;
@class MSStoreLoader;
@class MSLazyObject;


// Types of generated commands
typedef enum {
   kMSGeneratedCommandTypeApplication, 
   kMSGeneratedCommandTypeBookmark, 
   kMSGeneratedCommandTypeWorkflow
} MSGeneratedCommandType;


#pragma mark

@interface MSCommandsManager : NSObject {
   NSManagedObjectContext *workMoc;
   NSManagedObjectContext *workMoc2;
   NSManagedObjectContext *originalMoc;
   
   id inMemoryStore;
   
   NSMutableDictionary *roStoreLoaders;
   MSStoreLoader *userStoreLoader;
   MSSystemMonitor *storeMonitor;
   NSSet *vendors;
   NSSet *appBundles;
   
   NSManagedObjectContext *bookmarkContext;
   NSMutableDictionary *commandGroupTemplates;
   
   NSString *currentSpokenLanguage;
   NSString *currentOsLanguage;
   NSString *altLanguage;
   
   NSMutableDictionary *commandTrackerDictionary;

   NSString *activeAppBundleID;
   NSString *activeAppName;
   NSString *activeAppShortVersion;
   int activeAppMajorVersion;
   MSLocale *currentLocale;

   NSArray *currentGroups;
   unsigned int groupCount;
   MSCommandContext *currentContext;
	NSArray *availableCommands;
	unsigned int availableCommandsCount;
	
	NSMutableDictionary *usageNotes;
  NSMetadataQuery *bookmarkQuery;
}

#pragma mark Initialization
+ (MSCommandsManager *)sharedManager;
- (void)reset;
- (void)migrateROStoresToModel2;
- (NSArray *)migrateCommands:(NSArray *)commands fromPath:(NSString *)path intoContext:(NSManagedObjectContext *)moc;

#pragma mark
#pragma mark Action Methods
- (void)refreshVendorAndBundleLists;
- (void)refreshAllCommands;
- (void)refreshCurrentCommandActions;
- (void)refreshCurrentCommandTriggers;
- (void)refreshCurrentGroups;
- (void)refreshCurrentContext;
- (CommandMO *)addUserCommand:(NSDictionary *)commandInfo;
//- (CommandMO *)addUserCommandNamed:(NSString *)name type:(NSString *)type description:(NSString *)description data:(NSData *)data text:(NSString *)text;
- (CommandMO *)addTemporaryCommandNamed:(NSString *)name withKey:(NSString *)key;
- (void)removeTemporaryCommandKey:(NSString *)commandKey;
- (CommandMO *)temporaryCommandForCommandKey:(NSString *)commandKey;
- (void)removeCommand:(CommandMO *)command;
- (void)removeCommands:(NSArray *)commands;
- (BOOL)saveUserCommands:(NSError **)error;
- (BOOL)save:(NSError **)error andReset:(BOOL)reset;
- (void)setVisibility:(BOOL)visibility forAllCommandsWithKey:(NSString *)key;
- (void)deleteHiddenCommandsForApplication:(NSDictionary *)applicationInformation;

#pragma mark
#pragma mark Command Loading and Unloading
- (BOOL)reloadUserCommands;
- (BOOL)loadUserCommandsAtPath:(NSString *)commandPath ofType:(NSString *)storeType;
- (int)loadCommandsAtPath:(NSString *)commandPath ofType:(NSString *)storeType;
- (int)loadCommandsAtPath:(NSString *)commandPath ofType:(NSString *)storeType recursive:(BOOL)recursive;
- (BOOL)unloadUserCommandsAtPath:(NSString *)commandPath ofType:(NSString *)storeType;
- (int)unloadCommandsAtPath:(NSString *)commandPath persist:(BOOL)persist;
- (int)unloadCommandsAtPath:(NSString *)commandPath persist:(BOOL)persist recursive:(BOOL)recursive;
- (BOOL)loadGeneratedCommandsOfType:(MSGeneratedCommandType)commandType;
- (void)unloadGeneratedCommandsOfType:(MSGeneratedCommandType)commandType;
- (void)persistAllROStoresWithReload:(BOOL)reload;
- (NSArray *)loadedStores;
- (NSDictionary *)infoForStorePath:(NSString *)storePath;
- (NSArray *)launchableApplications;

#pragma mark
#pragma mark Usage Notes
- (void)addUsageNoteNamed:(NSString *)noteName withKey:(NSString *)usageKey description:(NSString *)usageDesc notes:(NSArray *)notes;
- (void)addUsageNoteNamed:(NSString *)noteName withKey:(NSString *)usageKey description:(NSString *)usageDesc notes:(NSArray *)notes target:(NSObject *)target;
- (void)removeUsageNoteWithKey:(NSString *)usageKey;
- (NSArray *)sortedUsageNoteKeys;
- (MSCommandGroup *)usageNoteForKey:(NSString *)usageKey;

#pragma mark
#pragma mark Command Exporting
- (BOOL)exportCommands:(NSArray *)commands asVendor:(NSString *)vendor toStoreAtPath:(NSString *)storePath storeType:(NSString *)storeType error:(NSError **)error;
- (BOOL)exportCommands:(NSArray *)commands toStoreAtPath:(NSString *)storePath storeType:(NSString *)storeType error:(NSError **)error;
- (BOOL)exportCommandKeys:(NSSet *)selectedKeys withOriginalVendor:(BOOL)preserveVendor toStoreAtPath:(NSString *)storePath storeType:(NSString *)storeType error:(NSError **)error;
- (BOOL)exportCommandKeys:(NSSet *)selectedKeys withOriginalVendor:(BOOL)preserveVendor language:(NSString *)language toStoreAtPath:(NSString *)storePath storeType:(NSString *)storeType error:(NSError **)error;
- (BOOL)xmlCommandKeys:(NSSet *)selectedKeys withOriginalVendor:(BOOL)preserveVendor language:(NSString *)language toFileAtPath:(NSString *)filePath error:(NSError **)error;

#pragma mark
#pragma mark Command Importing
- (NSArray *)importUserCommands:(NSArray *)commands error:(NSError **)error;
- (BOOL)importUserStoreAtPath:(NSString *)storePath storeType:(NSString *)storeType;

#pragma mark
#pragma mark Process Monitoring
- (BOOL)addMonitorForProcessID:(int)processID named:(NSString *)processName;
- (void)removeMonitorForProcessID:(int)processID;

#pragma mark
#pragma mark Accessors
+ (NSString *)defaultStoreType;
- (NSManagedObjectContext *)managedObjectContext;
- (NSSet *)vendors;
- (NSSet *)appBundles;
- (NSString *)currentSpokenLanguage;
- (void)setCurrentSpokenLanguage:(NSString *)spokenLanguage;
- (NSString *)currentOsLanguage;
- (void)setCurrentOsLanguage:(NSString *)osLanguage;
- (NSString *)altLanguage;
- (void)setAltLanguage:(NSString *)language;
- (MSLocale *)currentLocale;
- (MSCommandContext *)currentContext;
- (void)setCurrentContext:(MSCommandContext *)context;
- (NSArray *)currentGroups;
- (void)setCurrentGroups:(NSArray *)groups;
- (unsigned int)groupCount;
- (NSArray *)availableCommands;
- (void)setAvailableCommands:(NSArray *)commands;
- (unsigned int)availableCommandsCount;

#pragma mark
#pragma mark Command Retrieval
- (NSArray *)commandsWithCondition:(NSString *)condition error:(NSError **)error;
- (NSArray *)locationsWithCondition:(NSString *)condition error:(NSError **)error;
- (CommandMO *)internalCommandWithID:(int)commandID;

#pragma mark
#pragma mark Group Display Names
+ (NSString *)globalGroupName;
+ (NSString *)globalApplicationGroupName;
+ (NSString *)globalBookmarkGroupName;
+ (NSString *)globalGeneralGroupName;

#pragma mark
#pragma mark Framework Version Info
+ (int)majorVersion;
+ (int)minorVersion;
+ (int)buildNumber;
+ (NSString *)versionString;

#pragma mark
#pragma mark Logging
- (void)logCommands:(NSArray *)commands withDetails:(BOOL)withDetails;
- (void)logAllCommandsWithDetails:(BOOL)withDetails;
- (void)logAllCommandsForMoc:(NSManagedObjectContext *)moc withDetails:(BOOL)withDetails;
- (void)logAllStoreLoaders;
- (void)dumpUserStoreWithTitle:(NSString *)title;

- (void)dumpAllCommandTrackers;
- (void)dumpCommandTrackerForCommandKey:(NSString *)commandKey;
- (void)dumpCommandTrackerSummaryRequireUser:(BOOL)requireUser;

@end


extern NSString *MSCommandsManagerDidChangeCommandsNotification;
extern NSString *MSCommandsManagerDidLoadCommandsNotification;
extern NSString *MSCommandsManagerDidUnloadCommandsNotification;
extern NSString *MSCommandsManagerDidIgnoreCommandNotification;
extern NSString *MSCommandsManagerApplicationActivatedNotification;
extern NSString *MSCommandsManagerProcessTerminatedNotification;
extern NSString *MSCommandsManagerGroupChangedNotification;
extern NSString *MSCommandsManagerStartedCommandNotification;
extern NSString *MSCommandsManagerFinishedCommandNotification;

#pragma mark
#pragma mark Simple Logging Configuration
// For setting the Logging Level
typedef enum {
   kLogLevelAll, kLogLevelDebug, kLogLevelInfo, kLogLevelWarn, kLogLevelError, kLogLevelFatal, kLogLevelNone
} LogLevel;

BOOL MSLogInitProxy(LogLevel level);
void MSSetLogLevelProxy(LogLevel level);
